/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.fusion.modules.actions.portlets;

import java.util.Iterator;

import org.apache.jetspeed.Jetspeed;
import org.apache.jetspeed.components.portletentity.PortletEntityAccessComponent;
import org.apache.jetspeed.components.portletregistry.PortletRegistry;
import org.apache.jetspeed.modules.actions.controls.EndCustomize;
import org.apache.jetspeed.om.profile.Entry;
import org.apache.jetspeed.om.profile.Parameter;
import org.apache.jetspeed.om.profile.Portlets;
import org.apache.jetspeed.om.profile.Profile;
import org.apache.jetspeed.om.profile.psml.PsmlParameter;
import org.apache.jetspeed.om.registry.PortletEntry;
import org.apache.jetspeed.portal.Portlet;
import org.apache.jetspeed.services.PortletFactory;
import org.apache.jetspeed.services.Registry;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.jetspeed.services.resources.FusionResources;
import org.apache.jetspeed.services.rundata.JetspeedRunData;
import org.apache.pluto.om.entity.PortletEntity;
import org.apache.pluto.om.portlet.PortletDefinition;
import org.apache.turbine.util.RunData;

/**
 * This action must be invoked to clean up the customization
 * state and redirect the user to his portal hompage
 *
 * @author <a href="mailto:taylorl@apache.org">David Sean Taylor</a>
 */
public class FusionEndCustomize extends EndCustomize
{
    private static final JetspeedLogger log = JetspeedLogFactoryService.getLogger(FusionEndCustomize.class.getName());    
    private static final String PORTLET_APPLICATION_NAME = FusionResources.getString(FusionResources.PORTLET_APPLICATION_PARAMETER);
    private static final String PORTLET_DEFINITION_NAME = FusionResources.getString(FusionResources.PORTLET_DEFINITION_PARAMETER);
    private static final String JSR_PORTLET = FusionResources.getString(FusionResources.JSR168_PORTLET);
    private static final String J2_ENTITY = FusionResources.getString(FusionResources.J2_ENTITY);  
    
    public void doPerform(RunData data) throws Exception
    {
        Profile profile = ((JetspeedRunData) data).getCustomizedProfile();
        Portlets portlets = profile.getDocument().getPortlets();
        if (portlets == null)
        {
            return;
        }
        int count = 0;
        count = traverse(portlets, count);
        if (count > 0)
        {
            JetspeedRunData jrun = (JetspeedRunData)data;
            jrun.getCustomizedProfile().store();
            System.out.println("Stored profile");            
        }        
        super.doPerform(data);
        
    }
    
    private int traverse(Portlets portlets, int count)
    {
        // First let's add all the Entries in the current level
        Iterator eItr = portlets.getEntriesIterator();
        while(eItr.hasNext())
        {
            Entry entry =  (Entry)eItr.next();
            
            Portlet portlet = null;
            try
            {
                portlet = PortletFactory.getPortlet(entry);
            }
            catch (Exception e)
            {
                log.error("Could not create portlet for entry " + entry.getId(), e);                
                continue;
            }
            if (portlet == null)
            {
                log.error("Could not create portlet for entry " + entry.getId());
                continue;
            }   
            if (isFusionPortlet(portlet))
            {
                String pa = portlet.getPortletConfig().getInitParameter(PORTLET_APPLICATION_NAME, null);
                String pd = portlet.getPortletConfig().getInitParameter(PORTLET_DEFINITION_NAME, null);
                String registryKey = pa + "::" + pd;
                
                Parameter param = entry.getParameter(J2_ENTITY);
                if (param == null)
                {
                    PortletRegistry registryAccess =
                        (PortletRegistry)Jetspeed.getComponentManager().getComponent(PortletRegistry.class);
                    PortletEntityAccessComponent entityAccess =
                        (PortletEntityAccessComponent)Jetspeed.getComponentManager().getComponent(PortletEntityAccessComponent.class);
                    
                    PortletDefinition portletDef = registryAccess.getPortletDefinitionByUniqueName(registryKey);
                    if (null == portletDef)
                    {
                        log.error("Could not find portlet definition in registry for " + registryKey);
                        continue;
                    }
                    PortletEntity entity = entityAccess.newPortletEntityInstance(portletDef);
                    try
                    {
                        entityAccess.storePortletEntity(entity);                    
                        param = new PsmlParameter();
                        param.setName(J2_ENTITY);
                        param.setValue(entity.getId().toString());
                        entry.addParameter(param);
                        count++;                        
                    }
                    catch (Exception e)
                    {
                        log.error("Failed to store portlet entity: " + registryKey, e);
                        continue;
                    }
                }
            }
            // entries.add(new JetspeedPortletInstance(portlet, profile));
        }
        
        //Now if there are child levels, drill down recursively
        if(portlets.getPortletsCount() > 0)
        {
            Iterator pItr = portlets.getPortletsIterator();
            while(pItr.hasNext())
            {
                Portlets childPortlets = (Portlets)pItr.next();
                count += traverse(childPortlets, count);
            }            
        }        
        return count;
    }
    
    private boolean isFusionPortlet(Portlet entry)
    {
        // instanceof would sure be nice here, too bad about the wrappers
        PortletEntry pe = (PortletEntry)
            Registry.getEntry(Registry.PORTLET, entry.getName());
        if (pe == null)
        {
            return false;
        }
        String parent = pe.getParent();
        while (parent != null)
        {
            if (parent.equals(JSR_PORTLET))
            {
                return true;
            }
            pe = (PortletEntry)Registry.getEntry(Registry.PORTLET, parent);
            if (pe == null)
            {
                break;
            }
            parent = pe.getParent();                
        }        
        return false;
    }
}
