/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.fusion.services.fusion;

import javax.servlet.ServletConfig;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.PropertiesConfiguration;

import org.apache.jetspeed.components.ComponentManager;
import org.apache.jetspeed.components.portletregistry.PortletRegistry;
import org.apache.jetspeed.cps.CommonPortletServices;
import org.apache.jetspeed.om.common.portlet.MutablePortletApplication;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.pluto.om.portlet.PortletDefinition;
import org.apache.turbine.services.InitializationException;
import org.apache.turbine.services.TurbineBaseService;
import org.apache.turbine.services.TurbineServices;
import org.apache.turbine.services.resources.ResourceService;

// Fusion
import org.apache.jetspeed.Jetspeed;
import org.apache.jetspeed.engine.Engine;
import org.apache.jetspeed.engine.SpringEngine;
import org.apache.jetspeed.exception.JetspeedException;

/**
 * Fusion Service, this service fuses J2 into J1  
 *
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor</a>
 * @version $Id: FusionServiceImpl.java,v 1.5 2004/11/08 05:20:45 taylor Exp $
 */
public class FusionServiceImpl extends TurbineBaseService
        implements
            FusionService
{
    private static final JetspeedLogger log = JetspeedLogFactoryService.getLogger(FusionServiceImpl.class.getName());

    private final static String CONFIG_PROPERTIES_FILE = "properties.file";
    private final static String CONFIG_DEBUG = "debug";

    private boolean debug = false;
    private String propertiesFile = "/WEB-INF/conf/fusion.properties";
            
    protected CommonPortletServices cps = null;

    // Jetspeed engine
    private Engine jetspeed = null;
    
    /**
     * This is the early initialization method called by the
     * Turbine <code>Service</code> framework
     * @param conf The <code>ServletConfig</code>
     * @exception throws a <code>InitializationException</code> if the service
     * fails to initialize
     */
    public synchronized void init(ServletConfig conf) 
        throws InitializationException
    {
        // already initialized
        if (getInit()) return;
        
        System.out.println("Starting Up Fusion Service");

        // get configuration parameters from Jetspeed Resources
        ResourceService serviceConf = ((TurbineServices)TurbineServices.getInstance())
                                                     .getResources(FusionService.SERVICE_NAME);
        propertiesFile = serviceConf.getString( CONFIG_PROPERTIES_FILE, propertiesFile );
        debug = serviceConf.getBoolean(CONFIG_DEBUG, debug);
                
        Configuration properties;
        String applicationRoot;

        try
        {
            applicationRoot = conf.getServletContext().getRealPath("/");
            String realPath = conf.getServletContext().getRealPath(propertiesFile);
            properties = (Configuration) new PropertiesConfiguration(realPath);
            properties.setProperty(CommonPortletServices.APPLICATION_ROOT_KEY, applicationRoot);
    
            System.out.println("APP ROOT = " + applicationRoot);
            System.out.println("PROP FILE = " + realPath);
            
            startFusion(properties, applicationRoot, conf);
        }
        catch (Throwable t)
        {
            t.printStackTrace();
            log.error(t);
            throw new InitializationException("Configure CPS Exception", t);
        }
/*
        try
        {
            // Start the CPS Manager
            cps = CommonPortletServices.getInstance();
            cps.init(properties, applicationRoot);
            
        }
        catch (CPSInitializationException e)
        {
            e.printStackTrace();
            log.error(e);
            throw new InitializationException("Initialize CPS Exception", e);
        }
*/
        System.out.println("Fusion Service: Initialization Done");
        // initialization done
        setInit(true);
    }

    public void shutdown()
    {
        System.out.println("Shutting down CPS Bridge");
        Engine engine =Jetspeed.getEngine();
        try
        {
            engine.shutdown();
        }
        catch (JetspeedException e)
        {
            System.out.println("ERROR: Shutting down CPS Bridge " + e);
        }
        System.out.println("D0NE: Shutting down CPS Bridge");
        
    }

    /**
     * Fuse in J2
     * 
     * @param configuration The properties file for J2
     * @param applicationRoot The application root
     * @param servletConfig 
     * @throws InitializationException
     */
    public void startFusion(Configuration configuration, String applicationRoot, ServletConfig servletConfig)
        throws InitializationException
    {
        try
        {
            // Start Jetspeed
            Engine engine = Jetspeed.createEngine(configuration, applicationRoot, servletConfig, SpringEngine.class);
            engine.initContainer(servletConfig);
            
        }
        catch (Exception e)
        {
            throw new InitializationException("Failed to initialize Fusione engine", e);
        }        
    }
    
    private void testRegistryConnection()
    throws InitializationException
{
    try
    {
        ComponentManager cm = Jetspeed.getComponentManager();
        PortletRegistry registry = (PortletRegistry)cm.getComponent(PortletRegistry.class);
        if (registry == null) 
        {
            System.err.println("Failed to get registry component!");
            return;
        }

        MutablePortletApplication pac = registry.getPortletApplication("jetspeed");
        if (pac != null)
        {
            PortletDefinition portlet = pac.getPortletDefinitionByName("VelocityTwoColumns");
            if (portlet != null)
            {
                System.out.println("REGISTRY TEST [READ PORTLET] ENTRY. Name[jetspeed] sucessful!");
            }
            else
            {
                throw new InitializationException(
                                        "Registry Test: Load Portlet failed!, Portlet doesn't exist");
                
            }
        }
        else
        {
            throw new InitializationException(
                                    "Registry Test: Load Portlet Application failed!, Application doesn't exist");
       }
    }
    catch(Throwable e)
    {
        System.out.println("REGISTRY TEST [READ PORTLET] ENTRY. Name[jetspeed] failed. Error[" + e + "]");
        e.printStackTrace();
        throw new InitializationException("JetspeedPortletRegistry.getPortletApplication failed" +
                                 "Trace[" + e + "]", e);
    }
    
}    
}
