/* Copyright 2004 Apache Software Foundation
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.apache.jetspeed.fusion.userinfo.impl;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.jetspeed.components.portletregistry.PortletRegistry;
import org.apache.jetspeed.om.common.UserAttributeRef;
import org.apache.jetspeed.om.common.portlet.MutablePortletApplication;
import org.apache.jetspeed.om.security.JetspeedUser;
import org.apache.jetspeed.request.RequestContext;
import org.apache.jetspeed.services.rundata.JetspeedRunData;
import org.apache.jetspeed.services.rundata.JetspeedRunDataService;
import org.apache.jetspeed.userinfo.UserInfoManager;
import org.apache.jetspeed.userinfo.impl.AbstractUserInfoManagerImpl;
import org.apache.pluto.om.common.ObjectID;
import org.apache.turbine.services.rundata.RunDataService;
import org.apache.turbine.services.TurbineServices;


/**
 * <p>
 * Implements the {@link org.apache.jetspeed.userinfo.UserInfoManager}
 * interface with a J1/Turbine (JetspeedUser) mapping to JSR 168 User Attributes
 * See Portlet API Specification -- PLT.D Appendix for details on User Attribute naming.
 * </p>
 * 
 * @author <a href="mailto:taylorapache.org">David Sean Taylor</a>
 * @version $Id: FusionUserInfoManagerImpl.java,v 1.3 2004/11/08 05:20:45 taylor Exp $
 */
public class FusionUserInfoManagerImpl extends AbstractUserInfoManagerImpl implements UserInfoManager
{
    private static final Log log = LogFactory.getLog(FusionUserInfoManagerImpl.class);
    
    private final static String FUSION_USER_INFO = "org.apache.jetspeed.fusion.userinfo";
    
    private JetspeedRunDataService runDataService = null;
    private PortletRegistry registry;
    
    public FusionUserInfoManagerImpl(PortletRegistry registry)
    {
        this.registry = registry;
    }
    
    /* (non-Javadoc)
     * @see org.apache.jetspeed.userinfo.UserInfoManager#getUserInfoMap(org.apache.pluto.om.common.ObjectID, org.apache.jetspeed.request.RequestContext)
     */
    public Map getUserInfoMap(ObjectID oid, RequestContext context)
    {
        JetspeedRunData rundata = getRunDataService().getCurrentRunData();
        
        JetspeedUser user = rundata.getJetspeedUser();
        if (user == null)
        {
            return null;
        }
        
        Map map = (Map)context.getAttribute(FUSION_USER_INFO);
        if (map == null)
        {            
            Map j1Map = createJetspeedUserMap(user);
            MutablePortletApplication pa = registry.getPortletApplication(oid);
            if (null == pa)
            {
                return null;
            }
            Collection userAttributes = pa.getUserAttributes();
            Collection userAttributeRefs = pa.getUserAttributeRefs();
            map = mapUserInfo(j1Map, userAttributes, userAttributeRefs);
            
        }
        return map;
    }

    /**
     * <p>
     * Maps the user info properties retrieved from the user preferences to the
     * user info attribute declared in the portlet.xml descriptor.
     * </p>
     * 
     * @param j1Map
     *            The J1 user attributes.
     * @param userAttributes
     *            The declarative portlet user attributes.
     * @param userAttributeRefs
     *            The declarative jetspeed portlet extension user attributes
     *            reference.
     * @return The user info map.
     */
    private Map mapUserInfo(Map j1Map, Collection userAttributes, Collection userAttributeRefs)
    {
        if ((null == userAttributes) || (userAttributes.size() == 0))
        {
            return null;
        }

        Map userInfoMap = new HashMap();
        Object[] propertyKeys = j1Map.keySet().toArray();

        Collection linkedUserAttributes = mapLinkedUserAttributes(userAttributes, userAttributeRefs);
        Iterator iter = linkedUserAttributes.iterator();
        while (iter.hasNext())
        {
            UserAttributeRef currentAttributeRef = (UserAttributeRef) iter.next();
            if (null != currentAttributeRef)
            {
                for (int ix = 0; ix < propertyKeys.length; ix++)
                {
                    if (null != currentAttributeRef.getNameLink())
                    {
                        if ((currentAttributeRef.getNameLink()).equals(propertyKeys[ix]))
                        {
                            userInfoMap.put(currentAttributeRef.getName(), j1Map.get(propertyKeys[ix]));
                        }
                    }
                    else
                    {
                        if ((currentAttributeRef.getName()).equals(propertyKeys[ix]))
                        {
                            userInfoMap.put(currentAttributeRef.getName(), j1Map.get(propertyKeys[ix]));
                        }
                    }
                }
            }
        }

        return userInfoMap;
    }
    
    
    /**
     * Get the Jetspeed User Map -- override this to support extended Jetspeed User info
     * 
     * @param user
     * @return map of jetspeed user attributes mapped to portlet api user attributes
     */
    protected Map createJetspeedUserMap(JetspeedUser user)
    {
        Map map = new HashMap();
        map.put("user.name.given", user.getFirstName());
        map.put("user.name.family", user.getLastName());
        map.put("user.home-info.online.email", user.getEmail());
        map.put("user.business-info.online.email", user.getEmail());
        Iterator props = user.getPermStorage().entrySet().iterator();
        while (props.hasNext())
        {
            Map.Entry entry = (Map.Entry)props.next();
            if (entry.getKey().toString().startsWith("user."))
            {
                map.put(entry.getKey(), entry.getValue());
            }                
        }
        return map;
    }
    
    private JetspeedRunDataService getRunDataService()
    {
        if (runDataService == null)
        {
            runDataService =
                (JetspeedRunDataService) TurbineServices.getInstance().getService(RunDataService.SERVICE_NAME);
        }
        return runDataService;
    }
}
