/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jetspeed.modules.parameters;

//ecs stuff
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.jetspeed.util.StringUtils;
import org.apache.turbine.util.RunData;
import org.apache.velocity.context.Context;

/**
 *  Returns a group of check boxes using the following options:
 *  <UL>
 *  <LI><code>items</code>: list of comma-delimited check box names/values</LI>
 *  <LI><code>layout</code> [$northsouth|<strong>$eastwest</strong>]: presentation layout</LI>
 *  <LI><code>prefix</code>: prefix to use for check box names, default="cb"</LI> 
 * </UL>
 * 
 * @author <a href="mailto:mark_orciuch@ngsltd.com">Mark Orciuch</a>
 * @version $Id: CheckBoxGroup.java,v 1.5 2005/01/18 17:08:45 morciuch Exp $
 */
public class CheckBoxGroup extends VelocityParameterPresentationStyle
{
	private static final JetspeedLogger logger =
		JetspeedLogFactoryService.getLogger(CheckBoxGroup.class.getName());

	public static final String ITEMS = "items";
	public static final String LAYOUT = "layout";
	public static final String LAYOUT_EW = "eastwest";
	public static final String LAYOUT_NS = "northsouth";
	public static final String PREFIX = "prefix";
	public static final String PREFIX_DEFAULT = "cb";
	public static final String DISABLED = "disabled";
	public static final String SORTED = "sorted";
	public static final String ONCLICK_JAVASCRIPTS = "onclick_javascripts";

	private String items = null;
	private String layout = null;
	private String prefix = null;
	private String disabled = null;
	private boolean sorted = false;

	/**
	 * Put custom objects in velocity context
	 * 
	 * @param data
	 * @param name
	 * @param value
	 * @param parms
	 * @param context
	 * @return string
	 */
	public void buildContext(
		RunData data,
		String name,
		String value,
		Map parms,
		Context context)
	{
		try
		{
			init(data);

			Map itemMap = getListItems(data, name, value, parms, context);
			Map oc_js =
				getOnClickJavaScripts(data, name, value, parms, context, itemMap);

			context.put(ITEMS, itemMap);
			context.put(ONCLICK_JAVASCRIPTS, oc_js);
			context.put(LAYOUT, this.layout);
			context.put(PREFIX, this.prefix);
		}
		catch (Exception e)
		{
			logger.error(e);
			e.printStackTrace();
		}
	}

	/**
	 * Returns a map of list items (key=value)
	 * 
	 * @return
	 */
	protected Map getListItems(
		RunData data,
		String name,
		String value,
		Map parms,
		Context context)
	{
		if (items.indexOf("=") >= 0 && items.indexOf(",") >= 0)
		{
			return StringUtils.splitStringPairs(items, ",", "=", sorted);
		}
		else
		{
			Map result = new Hashtable();
			List list = StringUtils.splitString(items, ",");
			for (Iterator it = list.iterator(); it.hasNext();)
			{
				String item = (String) it.next();
				result.put(item, item);
			}
			return result;
		}
	}

	/**
	 * Initialize options
	 * 
	 * @param data
	 */
	protected void init(RunData data) throws Exception
	{
		sorted =
			new Boolean((String) this.getParm(SORTED, "false")).booleanValue();
		layout = (String) this.getParm(LAYOUT, LAYOUT_EW);
		prefix = (String) this.getParm(PREFIX, PREFIX_DEFAULT);
		disabled = (String) this.getParm(DISABLED, "false");
		items = (String) this.getParm(ITEMS, null);
	}

	/**
	 * @param data
	 * @param name
	 * @param value
	 * @param parms
	 * @param context
	 * @return
	 */
	private Map getOnClickJavaScripts(
		RunData data,
		String name,
		String value,
		Map parms,
		Context context,
		Map itemMap)
	{
		Vector keys = new Vector();
		for (Iterator it = itemMap.keySet().iterator(); it.hasNext();)
		{
			String key = (String) it.next();
			keys.add(key);
		}
        
		Map result = new Hashtable();
		for (Iterator it = itemMap.keySet().iterator(); it.hasNext();)
		{
			String key = (String) it.next();
			String javascript = getJavaScript(name, keys);            
			result.put(key, javascript);
		}
        
		return result;
	}

	/**
	 * 
	 * @param name
	 * @param v
	 * @return string
	 */
	private String getJavaScript(String name, Vector v)
	{
		StringBuffer result = new StringBuffer();
		result.append(name).append(".value = ");

		for ( Enumeration e = v.elements(); e.hasMoreElements(); )
		{
			String item = prefix + (String)e.nextElement();
			result.append("((");
			result.append(item);
			result.append(".checked) ? ");
			result.append(item);
			result.append(".value + \',\': '')");
			if (e.hasMoreElements())
			{
			   result.append(" + "); 
			}
		}
        
		result.append("; ");
		result.append(name).append(".value = ");
		result.append(name).append(".value.substring(0,");
		result.append(name).append(".value.length-1);");

		return result.toString();
	}
}
