/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jetspeed.modules.parameters;

// ECS support
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.turbine.util.RunData;
import org.apache.velocity.context.Context;

/**
 *  Presentation method to show a date input field with a popup calendar. Calendar is accessed by clicking the icon
 *  next to input field. For now, the only date format supported is mm/dd/yyyy. If value <code>$today</code> is passed, the date in the input field will default to today's date.
 *  <p>Options:
 *  <UL>
 *  <LI><code>$<name>.style.formName</code> - form name where the control is displayed; default=DefaultCustomizer</LI>
 *  <LI><code>$<name>.style.format</code> - date format to use for return value; default=mm/dd/yyyy.
 * The date format can have three types of separators: hyphen(-), space( ), or slash(/), but must be consistent in their usage. E.g.
 * d/m/yyyy
 * The acceptable tokens are :
 *   <UL>
 *   <LI>d - day</LI>
 *   <LI>dd - day (padded with 0 if less than 10)</LI>
 *   <LI>m - month (in numbers)</LI>
 *   <LI>mm - month (in numbers, padded with 0 if less than 10)</LI>
 *   <LI>mmm - month (in words)</LI>
 *   <li>yyyy - year</LI>
 *   </UL> 
 *  </LI>
 *  </UL>
 *  </p>
 * 
 * @author <a href="morciuch@apache.org">Mark Orciuch</a>
 * @version $Id: PopupCalendar.java,v 1.4 2005/01/18 17:08:45 morciuch Exp $
 */
public class PopupCalendar extends org.apache.jetspeed.modules.parameters.VelocityParameterPresentationStyle
{
	/**
	 * Static initialization of the logger for this class
	 */
	protected static final JetspeedLogger logger =
		JetspeedLogFactoryService.getLogger(
	PopupCalendar.class.getName());
			
	public static final String PARM_FORM_NAME = "formName";
	public static final String PARM_FORMAT = "format";
	public static final String PARM_INPUT_FORMAT = "input-format";    
	public static final String PARM_LAYOUT = "layout";
	public static final String LAYOUT_LABEL = "label";
	public static final String LAYOUT_INPUT = "input";
	public static final String DEF_INPUT_FORMAT = "yyyy-MM-dd hh:mm:ss.S";

	/**
	 * Put custom objects in velocity context
	 * 
	 * @param data
	 * @param name
	 * @param value
	 * @param parms
	 * @param context
	 * @return string
	 */
	public void buildContext(
		RunData data,
		String name,
		String value,
		Map parms,
		Context context)
	{

		String formName =
			(String) this.getParm(PARM_FORM_NAME, "DefaultCustomizer");
		context.put(PARM_FORM_NAME, formName);            
		String pattern = (String) this.getParm(PARM_FORMAT, "MMM d, yyyy");
		context.put(PARM_FORMAT, pattern);
		String layout = (String) this.getParm(PARM_LAYOUT, LAYOUT_INPUT);
		context.put(PARM_LAYOUT, layout);
		String inputFormat = (String) this.getParm(PARM_INPUT_FORMAT, DEF_INPUT_FORMAT);

		Date dt = null;
		if (value.equals("$today"))
		{
			dt = new Date(System.currentTimeMillis());
			value = new SimpleDateFormat(pattern).format(dt);
		}
		else
		{
			try
			{
				if (StringUtils.isNotEmpty(value))
				{
					dt = new SimpleDateFormat(inputFormat).parse(value);
				}
				else
				{
					dt = new Date();
				}
				value = new SimpleDateFormat(pattern).format(dt);
			}
			catch (Exception e)
			{
				logger.warn("Error parsing date: " + value, e);
			}
		}

		context.put("value", value);
		context.put("name", name);        
	}
}
