/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.om.dbregistry;

import java.sql.Connection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.jetspeed.om.BaseSecurityReference;
import org.apache.jetspeed.om.SecurityReference;
import org.apache.jetspeed.om.registry.CachedParameter;
import org.apache.jetspeed.om.registry.Category;
import org.apache.jetspeed.om.registry.MetaInfo;
import org.apache.jetspeed.om.registry.PortletEntry;
import org.apache.jetspeed.om.registry.RegistryException;
import org.apache.jetspeed.om.registry.base.BaseCachedParameter;
import org.apache.jetspeed.om.registry.base.BaseMetaInfo;
import org.apache.jetspeed.om.registry.base.BasePortletEntry;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.torque.Torque;
import org.apache.torque.TorqueException;
import org.apache.torque.util.Criteria;

/**
 * Implements the Portlet Registry database operations
 * 
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor </a>
 * @version $Id: PortletDbEntryPeer.java,v 1.4 2004/08/29 22:35:39 jford Exp $
 */
public class PortletDbEntryPeer extends
        org.apache.jetspeed.om.dbregistry.BasePortletDbEntryPeer
{

    private static final JetspeedLogger logger = JetspeedLogFactoryService
            .getLogger(PortletDbEntryPeer.class.getName());

    public static PortletEntry lookupPortletEntry(String entryName)
    {
        PortletEntry pe = null;
        try
        {
            PortletDbEntry pde = lookup(entryName);
            if (pde != null)
            {
                pe = mapDatabaseToEntry(pde);
            }
        } catch (Exception e)
        {
            logger.error("Exception retrieving Portlet Entry: " + entryName, e);
        }
        return pe;
    }

    private static PortletDbEntry lookup(String entryName)
            throws TorqueException
    {
        Criteria criteria = new Criteria();
        criteria.add(PortletDbEntryPeer.NAME, entryName);
        List result = PortletDbEntryPeer.doSelect(criteria);
        if (null == result || result.isEmpty()) 
        { 
            return null; 
        }
        return (PortletDbEntry) result.get(0);
    }

    public static PortletEntry mapDatabaseToEntry(PortletDbEntry pde)
            throws TorqueException
    {
        PortletEntry pe = new BasePortletEntry(pde.getId());
        pe.setApplication(pde.getApplication());
        pe.setCachedOnURL(pde.getCachedOnUrl());
        pe.setClassname(pde.getClassname());
        pe.setDescription(pde.getDescription());
        pe.setHidden(pde.getHidden());
        MetaInfo meta = new BaseMetaInfo();
        meta.setImage(pde.getImage());
        meta.setDescription(pde.getDescription());
        meta.setTitle(pde.getTitle());
        pe.setMetaInfo(meta);                
        pe.setName(pde.getName());            
        pe.setParent(pde.getParent());
        if(pde.getSecurityRef() != null)
        {
            SecurityReference ref = new BaseSecurityReference();
            ref.setParent(pde.getSecurityRef());
            pe.setSecurityRef(ref);
        }

        pe.setTitle(pde.getTitle());
        pe.setType(pde.getType());
        pe.setURL(pde.getURL());
        
        // parameters
        Iterator params = pde.getPortletParameters().iterator();
        while (params.hasNext())
        {
            PortletParameter pp = (PortletParameter)params.next();
            CachedParameter cp = new BaseCachedParameter(pp.getId());
            cp.setCachedOnName(pp.getCachedOnName());
            cp.setCachedOnValue(pp.getCachedOnValue());
            cp.setDescription(pp.getDescription());
            cp.setHidden(pp.getHidden());
            cp.setName(pp.getName());
            cp.setType(pp.getType());
            cp.setValue(pp.getValue());

            
            cp.setTitle(pp.getTitle());
            if (pp.getDescription() != null || pp.getTitle() != null || pp.getImage() != null)
            {
                MetaInfo meta2 = new BaseMetaInfo();
                meta2.setImage(pp.getImage());
                meta2.setDescription(pp.getDescription());
                meta2.setTitle(pp.getTitle());
                cp.setMetaInfo(meta2);
            }
            if (pp.getSecurityRef() != null)
            {
                SecurityReference ref2 = new BaseSecurityReference();
                ref2.setParent(pp.getSecurityRef());
                cp.setSecurityRef(ref2);
            }            
            pe.addParameter(cp);
        }
        
        // media types
        Iterator mtypes = pde.getPortletMediatypes().iterator();
        while (mtypes.hasNext())
        {
            PortletMediatype pm = (PortletMediatype)mtypes.next();
            pe.addMediaType(pm.getName());
        }

        // categories
        Iterator categories = pde.getPortletCategorys().iterator();
        while (categories.hasNext())
        {
            PortletCategory pc = (PortletCategory)categories.next();            
            pe.addCategory(pc.getName(), pc.getGroup());
        }
        
        return pe;
    }

    public static void removePortletSubObjects(long id)
            throws RegistryException
    {
        try
        {
            Criteria criteria = new Criteria();
            criteria.add(PortletParameterPeer.PORTLET_ID, id);
            PortletParameterPeer.doDelete(criteria);

            criteria = new Criteria();
            criteria.add(PortletMediatypePeer.PORTLET_ID, id);
            PortletMediatypePeer.doDelete(criteria);

            criteria = new Criteria();
            criteria.add(PortletCategoryPeer.PORTLET_ID, id);
            PortletCategoryPeer.doDelete(criteria);            
        } 
        catch (TorqueException e)
        {
            logger.error("Exception Removing Portlet Entry Accesses: " + id, e);
            throw new RegistryException("Exception Removing Portlet Entry: "
                    + id + ", " + e.toString());
        }
    }

    public static List fetchExtent() throws RegistryException
    {
        List extent = new ArrayList();
        try
        {
            Iterator result = PortletDbEntryPeer.doSelect(new Criteria())
                    .iterator();
            while (result.hasNext())
            {
                PortletEntry se = 
                    mapDatabaseToEntry((PortletDbEntry) result.next());
                extent.add(se);
            }
        } catch (TorqueException e)
        {
            String msg = "Exception Fetching Extent for Portlet ";
            logger.error(msg, e);
            throw new RegistryException(msg + e);
        }
        return extent;
    }

    public static void storePortletEntry(PortletEntry pe)
            throws RegistryException
    {
        Connection conn = null;
        boolean autoCommit = false;
        try
        {
            
            // get fresh copy in case its changed on another node
            PortletDbEntry pde = lookup(pe.getName());
            if (null != pde)
            {
                PortletDbEntryPeer.removePortletSubObjects(pde.getId());
            } else
            {
                pde = new PortletDbEntry();
            }
            conn = Torque.getConnection(DATABASE_NAME);
            autoCommit = conn.getAutoCommit();
            conn.setAutoCommit(false);

            pde.setApplication(pe.isApplication());
            pde.setCachedOnUrl(pe.isCachedOnURL());
            pde.setClassname(pe.getClassname());
            pde.setDescription(pe.getDescription());
            pde.setHidden(pe.isHidden());
            if (pe.getMetaInfo() != null)
            {
                pde.setImage(pe.getMetaInfo().getImage());
            }
            pde.setName(pe.getName());            
            pde.setParent(pe.getParent());
            if (pe.getSecurityRef() != null)
            {
                pde.setSecurityRef(pe.getSecurityRef().getParent());
            }
            pde.setTitle(pe.getTitle());
            pde.setType(pe.getType());
            pde.setURL(pe.getURL());
            
            pde.save(conn);

            // parameters
            Iterator parameters = pe.getParameterNames();            
            while (parameters.hasNext())
            {
                String name = (String)parameters.next();
                CachedParameter cp = (CachedParameter)pe.getParameter(name);

                PortletParameter pp = new PortletParameter();
                pp.setCachedOnName(cp.isCachedOnName());
                pp.setCachedOnValue(cp.isCachedOnValue());
                pp.setDescription(cp.getDescription());
                pp.setHidden(cp.isHidden());
                pp.setName(cp.getName());
                pp.setPortletId(pde.getId());
                pp.setTitle(cp.getTitle());
                pp.setType(cp.getType());
                pp.setValue(cp.getValue());
                
                MetaInfo meta = cp.getMetaInfo();
                if (meta != null)
                {
                    pp.setImage(meta.getImage());
                    pp.setDescription(meta.getDescription());
                    pp.setTitle(meta.getTitle());
                }
                SecurityReference ref = cp.getSecurityRef();
                if (ref != null)
                {
                    pp.setSecurityRef(ref.getParent());
                }
                pp.save(conn);
            }
            
            // media types
            Iterator mtypes = pe.listMediaTypes();            
            while (mtypes.hasNext())
            {
                String name = (String)mtypes.next();

                PortletMediatype pm = new PortletMediatype();
                pm.setName(name);
                pm.setPortletId(pde.getId());                
                pm.save(conn);
            }

            // categories
            Iterator categories = pe.listCategories();                        
            while (categories.hasNext())
            {
                Category category = (Category)categories.next();

                PortletCategory pc = new PortletCategory();
                pc.setName(category.getName());
                pc.setGroup(category.getGroup());
                pc.setPortletId(pde.getId());                
                pc.save(conn);
            }                        
        } 
        catch (Exception e)
        {
            try
            {
                conn.rollback();
            } catch (Exception e2)
            {
                logger.error("Failed to rollback", e2);
            }
            e.printStackTrace();
            logger.error("Exception storing Portlet Entry: " + pe.getName(), e);
            throw new RegistryException("Exception storing Portlet Entry: "
                    + pe.getName() + ", " + e.toString());
        } finally
        {
            // make sure to release the database connection
            Torque.closeConnection(conn);
            try
            {
                conn.setAutoCommit(autoCommit);
            } catch (Exception e)
            {
            }
        }

    }

    public static void removePortletEntry(String entryName)
            throws RegistryException
    {
        try
        {
            Criteria criteria = new Criteria();
            criteria.add(PortletDbEntryPeer.NAME, entryName);
            PortletDbEntryPeer.doDelete(criteria);
        } catch (TorqueException e)
        {
            logger.error("Exception Removing Portlet Entry: " + entryName, e);
            throw new RegistryException("Exception Removing Portlet Entry: "
                    + entryName + ", " + e.toString());
        }
    }

}