/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.om.dbregistry;

import java.sql.Connection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.jetspeed.om.registry.RegistryException;
import org.apache.jetspeed.om.registry.SecurityAccess;
import org.apache.jetspeed.om.registry.SecurityAllow;
import org.apache.jetspeed.om.registry.SecurityEntry;
import org.apache.jetspeed.om.registry.base.BaseSecurityAccess;
import org.apache.jetspeed.om.registry.base.BaseSecurityAllow;
import org.apache.jetspeed.om.registry.base.BaseSecurityAllowOwner;
import org.apache.jetspeed.om.registry.base.BaseSecurityEntry;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.torque.Torque;
import org.apache.torque.TorqueException;
import org.apache.torque.util.Criteria;


/**
 * Implements the Security Registry database operations
 *
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor</a>
 * @version $Id: SecurityDbEntryPeer.java,v 1.7 2004/07/22 00:47:03 jford Exp $
 */
public class SecurityDbEntryPeer
    extends org.apache.jetspeed.om.dbregistry.BaseSecurityDbEntryPeer
{
    public static final String ALLOW_OWNER = "owner";
    public static final String ALLOW_ROLE = "role";
    public static final String ALLOW_GROUP = "group";
    public static final String ALLOW_USER = "user";
    public static final String ALLOW_BOTH = "rolegroup";
    
    private static final JetspeedLogger logger = JetspeedLogFactoryService.getLogger(SecurityDbEntryPeer.class.getName());
    
    public static SecurityEntry lookupSecurityEntry(String entryName)
    {
        SecurityEntry se = null;
        try
        {
            SecurityDbEntry sde = lookup(entryName);
            if(sde != null)
            {
                se = mapDatabaseToEntry(sde);
            }
        }
        catch (Exception e)
        {
            logger.error("Exception retrieving Security Entry: " + entryName, e);
        }
        return se;
    }

    private static SecurityDbEntry lookup(String entryName)
    throws TorqueException
    {
        Criteria criteria = new Criteria();
        criteria.add(SecurityDbEntryPeer.NAME, entryName);
        List result = SecurityDbEntryPeer.doSelect(criteria);
        if (null == result || result.isEmpty())
        {
            return null;
        }            
        return (SecurityDbEntry)result.get(0);
    }
    
    
    public static void storeSecurityEntry(SecurityEntry se)
    throws RegistryException
    {
        Connection conn = null;
        boolean autoCommit = false;
        try
        {
            // get fresh copy in case its changed on another node
            SecurityDbEntry sde = lookup(se.getName());
            if (null != sde)
            {
                SecurityDbEntryPeer.removeSecurityEntryAccess(sde.getId());
            }
            else
            {
                sde = new SecurityDbEntry();
            }
            conn = Torque.getConnection(DATABASE_NAME);
            autoCommit = conn.getAutoCommit();
            conn.setAutoCommit(false);
                        
            sde.setName(se.getName());
            sde.setTitle(se.getTitle());
            sde.setDescription(se.getDescription());
            sde.save(conn);
            
            Iterator accesses = se.getAccesses().iterator();
            while (accesses.hasNext())
            {
                SecurityAccess access = (SecurityAccess)accesses.next();
                SecurityAccessDbEntry sade = new SecurityAccessDbEntry();                
                sade.setAction(access.getAction());
                sade.setEntryId(sde.getId());
                sade.save(conn);
                
                Iterator allows = access.getAllAllows().iterator();
                while (allows.hasNext())
                {
                    SecurityAllow allow = (SecurityAllow)allows.next();
                    SecurityAllowDbEntry sa = new SecurityAllowDbEntry();
                    if (allow.isOwner())
                    {
                        sa.setAllowType(ALLOW_OWNER);
                        sa.setAllowValue(ALLOW_OWNER);
                    }
                    else if (allow.getRole() != null && allow.getGroup() != null)
                    {
                        sa.setAllowType(ALLOW_BOTH);
                        sa.setAllowValue(allow.getRole());                        
                        sa.setAllowGroup(allow.getGroup());
                    }
                    else if (allow.getRole() != null)
                    {
                        sa.setAllowType(ALLOW_ROLE);
                        sa.setAllowValue(allow.getRole());                        
                    }
                    else if (allow.getGroup() != null)
                    {
                        sa.setAllowType(ALLOW_GROUP);
                        sa.setAllowValue(allow.getGroup());                        
                    }
                    else 
                    {
                        sa.setAllowType(ALLOW_USER);
                        sa.setAllowValue(allow.getUser());
                    }
                    sa.setAccessId(sade.getId());
                    sa.save(conn);    
                }                
            }            
       
        }
        catch (Exception e)
        {
            try
            {
                conn.rollback();
            }
            catch (Exception e2)
            {
                logger.error("Failed to rollback", e2);
            }
            logger.error("Exception storing Security Entry: " + se.getName(), e);
            throw new RegistryException("Exception storing Security Entry: " +  se.getName() + ", " + e.toString());
        }
        finally
        {
            // make sure to release the database connection
            Torque.closeConnection(conn);
            try
            {
                conn.setAutoCommit(autoCommit);
            }
            catch (Exception e)
            {
            }
        }
        
    }
    
    public static SecurityEntry mapDatabaseToEntry(SecurityDbEntry sde)
    throws TorqueException
    {
        SecurityEntry se = new BaseSecurityEntry(sde.getId());            
        se.setName(sde.getName());
        se.setTitle(sde.getTitle());
        se.setDescription(sde.getDescription());
        
        Iterator accesses = sde.getSecurityAccessDbEntrys().iterator();
        while (accesses.hasNext())
        {
            SecurityAccessDbEntry sade = (SecurityAccessDbEntry)accesses.next();
            SecurityAccess access = new BaseSecurityAccess(sade.getId());
            access.setAction(sade.getAction());
                            
            Iterator allows = sade.getSecurityAllowDbEntrys().iterator();
            while (allows.hasNext())
            {
                SecurityAllowDbEntry sa = (SecurityAllowDbEntry)allows.next();
                if (sa.getAllowType().equals(ALLOW_OWNER))
                {
                    SecurityAllow allow = new BaseSecurityAllowOwner(sa.getId());                    
                    allow.setOwner(true);
                    access.getOwnerAllows().add(allow);
                }
                else if (sa.getAllowType().equals(ALLOW_BOTH))
                {
                    SecurityAllow allow = new BaseSecurityAllow(sa.getId());
                    
                    allow.setRole(sa.getAllowValue());
                    allow.setGroup(sa.getAllowGroup());
                    access.getAllows().add(allow);                
                }
                else if (sa.getAllowType().equals(ALLOW_ROLE))
                {
                    SecurityAllow allow = new BaseSecurityAllow(sa.getId());
                    
                    allow.setRole(sa.getAllowValue());
                    access.getAllows().add(allow);
                }
                else if (sa.getAllowType().equals(ALLOW_GROUP))
                {
                    SecurityAllow allow = new BaseSecurityAllow(sa.getId());
                    
                    allow.setGroup(sa.getAllowValue());
                    access.getAllows().add(allow);
                }
                else 
                {
                    SecurityAllow allow = new BaseSecurityAllow(sa.getId());                    
                    allow.setUser(sa.getAllowValue());
                    access.getAllows().add(allow);
                }                    
            }
            se.getAccesses().add(access);
        }
        return se;
    }
    
    public static void removeSecurityEntry(String entryName)
    throws RegistryException    
    {
        try
        {
            Criteria criteria = new Criteria();
            criteria.add(SecurityDbEntryPeer.NAME, entryName);            
            SecurityDbEntryPeer.doDelete(criteria);
        }
        catch (TorqueException e)
        {
            logger.error("Exception Removing Security Entry: " + entryName, e);
            throw new RegistryException("Exception Removing Security Entry: " +  entryName + ", " + e.toString());
        }
    }

    public static void removeSecurityEntryAccess(long id)
    throws RegistryException        
    {
        try
        {
            Criteria criteria = new Criteria();
            criteria.add(SecurityAccessDbEntryPeer.ENTRY_ID, id);            
            SecurityDbEntryPeer.doDelete(criteria);
        }
        catch (TorqueException e)
        {
            logger.error("Exception Removing Security Entry Accesses: " + id, e);
            throw new RegistryException("Exception Removing Security Entry: " +  id + ", " + e.toString());
        }
    }
    
    public static List fetchExtent()
    throws RegistryException
    {
        List extent = new ArrayList();         
        try
        {
            Iterator result = SecurityDbEntryPeer.doSelect(new Criteria()).iterator();
            while (result.hasNext())
            {
                SecurityEntry se = mapDatabaseToEntry((SecurityDbEntry)result.next());
                extent.add(se);
            }
        }
        catch (TorqueException e)
        {
            String msg = "Exception Fetching Extent for Security ";
            logger.error(msg , e);
            throw new RegistryException(msg + e);
        }
        return extent;
    }
}
