/*
 * Copyright 2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.services.messaging.jms;

import java.util.Hashtable;
import java.util.Iterator;

import javax.jms.Connection;
import javax.jms.DeliveryMode;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.Topic;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;

import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.jetspeed.services.messaging.MessagingService;
import org.apache.turbine.services.InitializationException;
import org.apache.turbine.services.TurbineBaseService;

/**
 * This class provides basic functionality for accessing the JMS server.
 * This implementation conforms to the JMS 1.1 specification.
 * 
 * New implementations of a JMS Messaging Service should extend this
 * class and implement the createConnection method.  Also, any default
 * variables that need to be set should be added to a default constructor.
 * 
 * @author <a href="mailto:jford@apache.org">Jeremy Ford</a>
 * @version $Id: AbstractJMSMessagingService.java,v 1.3 2004/08/28 19:01:53 jford Exp $
 */
public abstract class AbstractJMSMessagingService extends TurbineBaseService
	implements MessagingService
{
    //static logger
    private static JetspeedLogger logger = JetspeedLogFactoryService.getLogger(AbstractJMSMessagingService.class.getName());
    
    //url configuration
    protected static final String CONFIG_SCHEME = "scheme";
    protected static final String CONFIG_HOST = "host";
    protected static final String CONFIG_PORT = "port";
    protected static final String CONFIG_NAME = "name";
    protected static final String CONFIG_URL = "url";
    
    protected static final String CONFIG_CONNECTION_FACTORY = "connection_factory";
    
    //Session properties
    protected static final String CONFIG_TRANSACTED = "transacted";
    protected static final String CONFIG_ACK_MODE = "ack_mode";
    
    //message properties
    protected static final String CONFIG_TOPIC = "topic_enabled";
    protected static final String CONFIG_DURABLE = "durable";
    protected static final String CONFIG_NOLOCAL = "nolocal";
    
    //producer properties
    protected static final String CONFIG_DELIVERY_MODE = "delivery_mode";
    protected static final String CONFIG_PRIORITY = "priority";
    protected static final String CONFIG_TIME_TO_LIVE = "time_to_live";

    //url defaults
    protected String DEFAULT_SCHEME;
    protected String DEFAULT_HOST;
    protected String DEFAULT_PORT;
    protected String DEFAULT_NAME = "";
    protected String DEFAULT_URL;
    
    protected String DEFAULT_CONNECTION_FACTORY;
    
    protected boolean DEFAULT_TRANSACTED = false;
    protected int DEFAULT_ACK_MODE  = Session.AUTO_ACKNOWLEDGE;
    protected boolean DEFAULT_TOPIC = true;
    protected boolean DEFAULT_DURABLE = false;
    protected boolean DEFAULT_NOLOCAL = true;
    
    protected int DEFAULT_DELIVERY_MODE = DeliveryMode.NON_PERSISTENT;
    protected int DEFAULT_PRIORITY = 0; //0<x<10
    protected int DEFAULT_TIME_TO_LIVE = 0;
    
    //Acknowledgement keys
    protected static final String AUTO_ACKNOWLEDGE = "auto";
    protected static final String CLIENT_ACKNOWLEDGE = "client";
    protected static final String DUPS_OK_ACKNOWLEDGE = "dups_ok";
    
    //Delivery Mode Key
    protected static final String PERSISTENT = "persistent";
    protected static final String NON_PERSISTENT = "non_persistent";
    
    //url properties
    protected String scheme;
    protected String host;
    protected String port;
    protected String name;
    protected String url;
    
    //connection factory properties
    protected String connectionFactory;
    protected String initialContextFactory;
    
    //default producer properties
    protected int deliveryMode;
    protected int priority;
    protected int timeToLive;

    //message properties
    protected boolean transacted = false;
    protected int ackMode;
    
    protected boolean use_topic;
    protected boolean durable;
    protected boolean nolocal;
    
    protected Hashtable destinations = new Hashtable();  //maps destination string to Destination object
    protected Hashtable consumers = new Hashtable();  //maps uniqueId to consumer
    protected Hashtable producers = new Hashtable();  //maps destination string to Producer object
    
    protected Connection connection;
    protected Session session;
    
    /**
     * This is the early initialization method called by the
     * Turbine <code>Service</code> framework
     */
    public synchronized void init(ServletConfig conf) throws InitializationException
    {
        try
        {
            initJMSConfiguration();
            
            createConnection();
            
            connection.start();
            
            createSession();
        }
        catch(JMSException e)
        {
            logger.error("Failure to intialize service due to JMS Exception.", e);
            throw new InitializationException(e.getMessage(), e);
        }
        catch(Exception e)
        {
            logger.error("Unknown exception caught while trying to initialize messaging service.", e);
            throw new InitializationException(e.getMessage(), e);
        }
    }
    
    /**
     * Initialize the JMS Configuration
     *
     */
    protected void initJMSConfiguration()
    {
        
        logger.info("Initializing JMS Configuration.");
        
        //initialize service properties
        scheme = getConfiguration().getString(CONFIG_SCHEME, DEFAULT_SCHEME);
        host = getConfiguration().getString(CONFIG_HOST, DEFAULT_HOST);
        port = getConfiguration().getString(CONFIG_PORT, DEFAULT_PORT);
        name = getConfiguration().getString(CONFIG_NAME, DEFAULT_NAME);
        url = getConfiguration().getString(CONFIG_URL, DEFAULT_URL);
        
        logger.info("JMS URL (Required) configured to " + this.getProviderURL());
        
        connectionFactory = getConfiguration().getString(CONFIG_CONNECTION_FACTORY, DEFAULT_CONNECTION_FACTORY);
        
        logger.info("JMS Connection Facotry (Optional): " + connectionFactory);
        
        transacted = getConfiguration().getBoolean(CONFIG_TRANSACTED, DEFAULT_TRANSACTED);
        
        logger.info("JMS Transacted: "  + (transacted ? "yes" : "no"));
        
        use_topic = getConfiguration().getBoolean(CONFIG_TOPIC, DEFAULT_TOPIC);
        durable = getConfiguration().getBoolean(CONFIG_DURABLE, DEFAULT_DURABLE);
        nolocal = getConfiguration().getBoolean(CONFIG_NOLOCAL, DEFAULT_NOLOCAL);
        
        logger.info("JMS Use Topic: " + (use_topic ? "yes" : "no"));
        logger.info("JMS Durable: " + (durable ? "yes" : "no"));
        logger.info("JMS NoLocal: " + (nolocal ? "yes" : "no"));
        
        ackMode = DEFAULT_ACK_MODE;
        String ackModeStr = getConfiguration().getString(CONFIG_ACK_MODE);
        if(ackModeStr != null)
        {
            if(ackModeStr.equals(AUTO_ACKNOWLEDGE))
            {
                ackMode = Session.AUTO_ACKNOWLEDGE;
            }
            else if(ackModeStr.equals(CLIENT_ACKNOWLEDGE))
            {
                ackMode = Session.CLIENT_ACKNOWLEDGE;
            }
            else if(ackModeStr.equals(DUPS_OK_ACKNOWLEDGE))
            {
                ackMode = Session.DUPS_OK_ACKNOWLEDGE;
            }
        }
        
        logger.info("JMS Acknowledgement Mode: " + printAckMode());
        
        //producer related config
        deliveryMode = DEFAULT_DELIVERY_MODE;
        String deliveryModeStr = getConfiguration().getString(CONFIG_DELIVERY_MODE);
        if(deliveryModeStr != null)
        {
	        if(deliveryModeStr.equals(PERSISTENT))
	        {
	            deliveryMode = DeliveryMode.PERSISTENT;
	        }
	        else
	        {
	            deliveryMode = DeliveryMode.NON_PERSISTENT;
	        }
        }
        
        logger.info("JMS Delivery Mode: " + printDeliveryMode());
        
        priority = getConfiguration().getInt(CONFIG_PRIORITY, DEFAULT_PRIORITY);
        timeToLive = getConfiguration().getInt(CONFIG_TIME_TO_LIVE, DEFAULT_TIME_TO_LIVE);
        
        logger.info("JMS Message Priority: " + priority);
        logger.info("JMS Message TimeToLIve: " + timeToLive);
        
        logger.info("JMS Initialization complete.");
    }
    
    /**
     * Helper method to print the acknowledgement mode
     * @return
     */
    protected String printAckMode()
    {
        String ackModeStr = AUTO_ACKNOWLEDGE;
        if(ackMode == Session.CLIENT_ACKNOWLEDGE)
        {
            ackModeStr = CLIENT_ACKNOWLEDGE;
        }
        else if(ackMode == Session.DUPS_OK_ACKNOWLEDGE)
        {
            ackModeStr = DUPS_OK_ACKNOWLEDGE;
        }
        
        return ackModeStr;
    }
    
    /**
     * Helper method to print the delivery mode
     * @return
     */
    protected String printDeliveryMode()
    {
        String deliveryModeStr = PERSISTENT;
        if(deliveryMode == DeliveryMode.NON_PERSISTENT)
        {
            deliveryModeStr = NON_PERSISTENT;
        }
        
        return deliveryModeStr;
    }
    
    /**
     * Return the url if it was configured, othewise, generate the url
     * from the scheme, host, port, and name properties
     * @return
     */
    protected String getProviderURL()
    {
        String providerURL = url;
        
        if(providerURL == null)
        {
            if((scheme != null) && (host != null) && (port != null) && (name != null))
            {
                providerURL = scheme + "://" + host + ":" + port + "/" + name;
            }
        }
        return providerURL;
    }
    
    /**
     * Genereate a JNDI context using the context factory and
     * the provider url
     * @return
     * @throws NamingException
     */
    protected Context getContext() throws NamingException
    {
        Hashtable properties = new Hashtable();
    	
        properties.put(
            Context.INITIAL_CONTEXT_FACTORY, 
            initialContextFactory);
    
        String providerURL = getProviderURL();
        properties.put(Context.PROVIDER_URL, providerURL);
        
        return new InitialContext(properties);
    }
    
    /**
     * Create the JMS Session
     * @throws JMSException
     */
    protected void createSession() throws JMSException
    {
        session = connection.createSession(transacted, ackMode);
    }
    
    /**
     * Create the connection the the JMS Server
     * @throws JMSException
     */
    protected abstract void createConnection() throws JMSException;
    
    
    /**
     * @see org.apache.jetspeed.services.messaging.MessagingService#createMessage(int)
     */
    public Message createMessage(int messageType)
    {
        Message message = null;
        
        try
        {
	        if(messageType == TEXT_MESSAGE)
	        {
	            message = session.createTextMessage();
	        }
	        else if(messageType == BYTE_MESSAGE)
	        {
	            message = session.createBytesMessage();
	        }
	        else if(messageType == MAP_MESSAGE)
	        {
	            message = session.createMapMessage();
	        }
	        else if(messageType == OBJECT_MESSAGE)
	        {
	            message = session.createObjectMessage();
	        }
	        else if(messageType == STREAM_MESSAGE)
	        {
	            message = session.createStreamMessage();
	        }
	        else
	        {
	            message = session.createMessage();
	        }
        }
        catch(JMSException e)
        {
            logger.error("Failure to create message", e);
        }
        
        return message;
    }
    
    /**
     * This is the shutdown method called by the
     * Turbine <code>Service</code> framework
     */
    public void shutdown()
    {
        Iterator keyIter = consumers.keySet().iterator();
        while (keyIter.hasNext())
        {
            String id = (String) keyIter.next();
            MessageConsumer consumer = (MessageConsumer)consumers.get(id);
            try
            {
                logger.info("Closing consumer " + id);
                consumer.close();
                logger.info("Consumer " + id + " closed successfully.");
            }
            catch (JMSException e)
            {
                logger.error("Failed to close consumer " + id, e);
            }
        }
        
        Iterator producerKeyIter = producers.keySet().iterator();
        while (producerKeyIter.hasNext())
        {
            String id = (String) producerKeyIter.next();
            MessageProducer producer = (MessageProducer)producers.get(id);
            try
            {
                logger.info("Closing producer to destination " + id);
                producer.close();
                logger.info("Producer " + id + " closed successfully.");
            }
            catch (JMSException e)
            {
                logger.error("Failed to closed producer " + id, e);
            }
        }
        
        try
        {
            logger.info("Closing session.");
            session.close();
            logger.info("Session closed.");
        }
        catch (JMSException e)
        {
            logger.error("Failed to close session", e);
        }
        try
        {
            logger.info("Closing JMS connection.");
            connection.stop();
            connection.close();
            logger.info("Connection closed.");
        }
        catch (JMSException e)
        {
            logger.error("Failed to close connection.", e);
        }
    }
    
    /**
     * @see org.apache.jetspeed.services.messaging.MessagingService#addMessageListener(java.lang.Object)
     */
    public void addMessageListener(MessageListener listener, String id, String destination)
    {
        try
        {
            Destination consumerDestination = getDestination(destination);
            MessageConsumer consumer = (MessageConsumer) consumers.get(id);
            if(consumer == null)
            {
	            if(this.use_topic && this.durable)
	            {
	                if(durable)
	                {
	                    consumer = session.createDurableSubscriber((Topic)consumerDestination, id, "", nolocal); //topic, uniqueID, selector, nolocal
	                }
	            }
	            else
	            {
	                consumer = session.createConsumer(consumerDestination, "", nolocal);
	            }
            }
            consumer.setMessageListener(listener);
            consumers.put(id, consumer);
        }
        catch (JMSException e)
        {
            logger.error("Failed to add message listener " + id + " to destination " + destination, e);
        }
    }
    
    
    /**
     * @see org.apache.jetspeed.services.messaging.MessagingService#createConsumer(java.lang.String)
     */
    public MessageConsumer createConsumer(String destination)
    {
        MessageConsumer consumer = null;
        try
        {
            Destination consumerDestination = getDestination(destination);
            if(consumerDestination == null)
            {
                logger.error("Failed to obtain destination " + destination + ".  A consumer cannot be created.");
            }
            else
            {
                logger.debug("Attempting to create consumer");
                consumer = session.createConsumer(consumerDestination);
                logger.debug("Consumer created successfully");
            }
        }
        catch (JMSException e)
        {
            logger.error("Failed to create consumer to destincation " + destination);
        }
        
        return consumer;
    }
    
    /**
     * @see org.apache.jetspeed.services.messaging.MessagingService#removeMessageListener(java.lang.String)
     */
    public void removeMessageListener(String id)
    {
        MessageConsumer consumer = (MessageConsumer)consumers.get(id);
        
        if(consumer == null)
        {
            logger.error("Attempted to remove consumer " + id + " but could not find it in cache.");
        }
        else
        {
	        if(durable && use_topic)
	        {
	            try
	            {
	                session.unsubscribe(id);
	            }
	            catch (JMSException e)
	            {
	                logger.error("Failed to unsubscribe durable consumer " + id, e);
	            }
	        }
	        try
	        {
	            consumer.close();
	        }
	        catch (JMSException e)
	        {
	            logger.error("Failed to close consumer " + id, e);
	        }
	        
	        consumers.remove(id);
        }
    }
    

    /**
     * @see org.apache.jetspeed.services.messaging.MessagingService#sendMessage(java.lang.Object)
     */
    public void sendMessage(Message message, String destination)
    {
        try
        {
            logger.debug("Sending message " + message + " to " + destination);
            Destination producerDestination = getDestination(destination);
            if(producerDestination == null)
            {
                logger.error("Failed to obtain destination " + destination + ".  Message cannot be sent.");
                return;
            }
            
            MessageProducer producer = getProducer(destination);
            if(producer == null)
            {
                logger.error("Failed to obtain producer for destination " + destination + ".  Message cannot be sent.");
                return;
            }
            
            producer.send(producerDestination, message);//, deliveryMode, priority, 0);
            logger.debug("Message " + message + " sent to " + destination + " successfully.");
        }
        catch (JMSException e)
        {
            logger.error("Failed to send message to destination " + destination, e);
        }
    }
    
    protected Destination getDestination(String destination)
    {
        logger.debug("Attempting to find destination " + destination + " in cache.");
        Destination jmsDestination = (Destination) destinations.get(destination);
        
        if(jmsDestination == null)
        {
            logger.debug("Failed to find destination " + destination + " in cache.  Will attempt to create.");
	        try
	        {
		        if (use_topic) {
		            jmsDestination = session.createTopic(destination);
				}
				else {
				    jmsDestination = session.createQueue(destination);
				}
		        
		        destinations.put(destination, jmsDestination);
		        logger.debug("Created destination successfully and cached.");
	        }
	        catch (JMSException e)
	        {
	            logger.error("Failed to create destination " + destination, e);
	        }
        }
        else
        {
            logger.debug("Found destination in cache.");
        }
        
        return jmsDestination;
    }
    
    /**
     * Return the producer based on the given destination.
     * This implemenation will cache the producers that are created for use later.
     * @param destination
     * @return
     */
    protected MessageProducer getProducer(String destination)
    {
        Destination producerDestination = getDestination(destination);
        if(producerDestination == null)
        {
            logger.error("Failed to obtain destination " + destination + ".  A producer cannot be created.");
            return null;
        }
        
        logger.debug("Attempting to find producer for destination " + destination + " in cache.");
        MessageProducer producer = (MessageProducer) producers.get(destination);
        if(producer == null)
        {
            try
            {
                logger.debug("Attempting to create producer.");
                producer = session.createProducer(producerDestination);
                producer.setDeliveryMode(this.deliveryMode);
                producer.setTimeToLive(this.timeToLive);
                producer.setPriority(this.priority);
                
                producers.put(destination, producer);
                logger.debug("Producer created successfully.");
            }
            catch (JMSException e)
            {
                logger.error("Failed to create producer", e);
            }   
        }
        else
        {
            logger.debug("Found producer in cache");
        }
        
        return producer;
    }
}
