/*
 * Copyright 2000-2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.services.registry;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.ObjectMessage;
import javax.servlet.ServletConfig;

import org.apache.jetspeed.om.dbregistry.PortletDbEntryPeer;
import org.apache.jetspeed.om.dbregistry.SecurityDbEntryPeer;
import org.apache.jetspeed.om.registry.PortletEntry;
import org.apache.jetspeed.om.registry.Registry;
import org.apache.jetspeed.om.registry.RegistryEntry;
import org.apache.jetspeed.om.registry.RegistryException;
import org.apache.jetspeed.om.registry.SecurityEntry;
import org.apache.jetspeed.om.registry.base.LocalRegistry;
import org.apache.jetspeed.services.Messenger;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.jetspeed.services.messaging.MessagingService;
import org.apache.turbine.services.InitializationException;
import org.apache.turbine.services.TurbineServices;

/**
 * Implements the Jetspeed Registry Service interface, with a hybrid of Database and File-based (Castor)
 * Registry storage. Allows user to store some registries in the database, and others on the file system.
 *
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor</a>
 * @version $Id: HybridRegistryService.java,v 1.12 2004/08/05 04:27:03 jford Exp $
 */
public class HybridRegistryService 
    extends 
        CastorRegistryService
    implements
        HybridRegistry
{
    /** registry type keyed list of entries */
    private Hashtable dbRegistries = new Hashtable();
    private static final JetspeedLogger logger = JetspeedLogFactoryService.getLogger(HybridRegistryService.class.getName());
    private boolean hybrid = true;
    private boolean replaceImportedEntries = false;
        
    
    /**
     * This is the early initialization method called by the
     * Turbine <code>Service</code> framework
     */
    public synchronized void init(ServletConfig conf) throws InitializationException
    {
        
        // get the list of managed Registries in the Database
        // need to do this first before init of Castor service
        // castor will start RegistryWatcher
        Iterator iterator = getConfiguration().getKeys("database.default");
        while (iterator.hasNext())
        {
            String key = (String)iterator.next();
            String name = key.substring(key.lastIndexOf(".") + 1);                
            Registry registry = createRegistry(name);
            dbRegistries.put(name, registry);
        }
        
        hybrid = getConfiguration().getBoolean("hybrid");        
        if (hybrid)
        {
            super.init(conf);
        }
        else
        {
            //check for messaging if we're not in hybrid mode
            this.enableMessaging = getConfiguration().getBoolean("enable_messaging", false);
            if(enableMessaging)
            {
                try
                {
                    TurbineServices.getInstance().initService(MessagingService.SERVICE_NAME, conf);
            
                    Messenger.addMessageListener(this, CastorRegistryService.class.getName(), MessagingService.REGISTRY_UPDATE_SUBJECT);
                }
                catch(Exception e)
                {
                    enableMessaging = false;
                    logger.warn("Messaging support has been disabled due to a failure to initialize the service or add a listener.");
                }
            }
        }

        try
        {
            setInit(true);
            
            replaceImportedEntries = getConfiguration().getBoolean("replace.imported.entries");

            // get the list of managed Registries in the Database            
            Iterator regIter = dbRegistries.keySet().iterator();
            while (regIter.hasNext())
            {
                String key = (String)regIter.next();
                Registry registry = (Registry) dbRegistries.get(key);
                loadCache(key, registry);
            }
        }
        catch (Throwable t)
        {
            setInit(false);
            t.printStackTrace();
            throw new InitializationException("Unable to initialize HybridRegistryService: " + t.toString());
        }
        setInit(true);        
    }
    
    public boolean getReplaceImportedEntries()
    {
        return this.replaceImportedEntries;
    }
    
    private void loadCache(String registryName, Registry registry)
    throws InitializationException
    {
        LocalRegistry local = (LocalRegistry)registry;
        try
        {
            if (registryName.equals(org.apache.jetspeed.services.Registry.SECURITY))
            {                
                Iterator extent = SecurityDbEntryPeer.fetchExtent().iterator();                
                while (extent.hasNext())
                {
                    SecurityEntry se = (SecurityEntry)extent.next();
                    local.addLocalEntry((RegistryEntry)se);                    
                }
            }
            else if (registryName.equals(org.apache.jetspeed.services.Registry.PORTLET))
            {
                System.out.println("Portlet: Loading cache for " + registryName);
                
                Iterator extent = PortletDbEntryPeer.fetchExtent().iterator();                
                while (extent.hasNext())
                {
                    PortletEntry pe = (PortletEntry)extent.next();
                    System.out.println("Portlet: Loading cache for " + pe.getName());
                    if (!local.hasEntry(pe.getName()))
                    {
                        local.addLocalEntry((RegistryEntry)pe);
                    }
                }
                System.out.println("Portlet: done Loading cache for " + registryName);
                
            }
        }
        catch (Throwable t)
        {
            t.printStackTrace();
            logger.error("Failed to load cache for registry: " + registryName, t);
            throw new InitializationException("Unable to initialize HybridRegistryService cache", t); 
        }
    }
    
    public Enumeration getNames()
    {
        ArrayList result = new ArrayList();
        if (hybrid)
        {
            Enumeration names = super.getNames();
            if (names != null)
            {
                result.addAll(Collections.list(names));            
            }
        }
        result.addAll(dbRegistries.keySet());
        return Collections.enumeration(result);        
    }

    public List listDatabaseRegistries()
    {
        ArrayList result = new ArrayList();
        result.addAll(dbRegistries.keySet());
        return result;
    }

    public List listFileRegistries()
    {
        ArrayList result = new ArrayList();
        if (hybrid)
        {
            Enumeration names = super.getNames();
            if (names != null)
            {
                result.addAll(Collections.list(names));            
            }
        }
        return result;                
    }
    
    
    public Registry get(String registryName)
    {
        Registry registry = (Registry)dbRegistries.get(registryName);
        if (null == registry && hybrid)
        {
            return super.get(registryName);
        }        
        return registry;
    }
    
    public RegistryEntry createEntry(String registryName)
    {
        Registry registry = (Registry)dbRegistries.get(registryName);
        if (null == registry)
        {
            if (hybrid)
            {
                return super.createEntry(registryName);
            }
            return null; 
        }            
        
        return registry.createEntry();
    }
    
    public RegistryEntry getEntry(String registryName, String entryName)
    {        
        RegistryEntry entry = null;
        Registry registry = (Registry)dbRegistries.get(registryName);
        if (null == registry)
        {            
            if (hybrid)
            {            
                return super.getEntry(registryName, entryName);
            }
            return null;
        }            
                
        try
        {
            entry = registry.getEntry(entryName);
        }
        catch (RegistryException iee)
        {
            if (logger.isInfoEnabled())
            {
                logger.info("HybridRegistryService: Failed to lookup " + entryName + " from " + registryName);
            }            
        }
        if (null == entry)
        {            
            if (registryName.equals(org.apache.jetspeed.services.Registry.SECURITY))
            {                                         
                entry = (RegistryEntry)SecurityDbEntryPeer.lookupSecurityEntry(entryName);
            }
            else if (registryName.equals(org.apache.jetspeed.services.Registry.PORTLET))
            {                                         
                entry = (RegistryEntry)PortletDbEntryPeer.lookupPortletEntry(entryName);
            }
            if (entry != null)
            {
                LocalRegistry local = (LocalRegistry)registry;
                try
                {
                    local.addLocalEntry(entry);
                }
                catch (RegistryException e)
                {
                    if (logger.isInfoEnabled())
                    {
                        logger.info("HybridRegistryService: Failed to add to local cache " + entryName + " from " + registryName);
                    }                                    
                }
            }                
        }
        else
        {
        }
        return entry;        
    }
    
    public void addEntry(String registryName, RegistryEntry entry)
    throws RegistryException
    {
        LocalRegistry registry = (LocalRegistry)dbRegistries.get(registryName);
        if (null == registry)
        {
            if (hybrid)
            {            
                super.addEntry(registryName, entry);
            }
            return;
        }            
        if (entry == null)
        {
            return;
        }        
        
        save(registry, entry);
        
        if(enableMessaging)
        {
            Message msg = createMessage(registryName, entry.getName());
            Messenger.sendMessage(msg, MessagingService.REGISTRY_UPDATE_SUBJECT);
        }
    }
    
    public void removeEntry(String registryName, String entryName)
    {
        try
        {
            LocalRegistry registry = (LocalRegistry)dbRegistries.get(registryName);
            if (null == registry)
            {
                if (hybrid)
                {
                    super.removeEntry(registryName, entryName);
                }
                return;            
            }                    
            if (entryName == null)
            {
                return;
            }
            if (registryName.equals(org.apache.jetspeed.services.Registry.SECURITY))
            {                                         
                    SecurityDbEntryPeer.removeSecurityEntry(entryName);        
                    registry.removeLocalEntry(entryName);
            }
            else if (registryName.equals(org.apache.jetspeed.services.Registry.PORTLET))
            {
                PortletDbEntryPeer.removePortletEntry(entryName);        
                registry.removeLocalEntry(entryName);                
            }
            
            Message msg = createMessage(registryName, entryName);
            Messenger.sendMessage(msg, MessagingService.REGISTRY_UPDATE_SUBJECT);
        }
        catch (RegistryException e)
        {
            
        }
    }

    public void saveEntry(String registryName, RegistryEntry entry) throws RegistryException
    {
        LocalRegistry registry = (LocalRegistry)dbRegistries.get(registryName);
        if (null == registry)
        {
            if (hybrid)
            {
                super.saveEntry(registryName, entry);
            }
            return;
        }
        if (entry == null)
        {
            return;
        }                
        save(registry, entry);
    }
    
    private void save(LocalRegistry registry, RegistryEntry entry) throws RegistryException
    {
        if (entry instanceof SecurityEntry)
        {
            SecurityEntry se = (SecurityEntry)entry;
            
            SecurityDbEntryPeer.storeSecurityEntry(se);
            se = SecurityDbEntryPeer.lookupSecurityEntry(se.getName());        
            if (registry.hasEntry(entry.getName()))
            {
                registry.setLocalEntry((RegistryEntry)se);
            }
            else
            {
                registry.addLocalEntry((RegistryEntry)se);
            }
        }
        else if (entry instanceof PortletEntry)
        {
            PortletEntry pe = (PortletEntry)entry;
            
            PortletDbEntryPeer.storePortletEntry(pe);
            pe = PortletDbEntryPeer.lookupPortletEntry(pe.getName());        
            if (registry.hasEntry(entry.getName()))
            {
                registry.setLocalEntry((RegistryEntry)pe);
            }
            else
            {
                registry.addLocalEntry((RegistryEntry)pe);
            }            
        }
    }
    
    protected void refresh(String regName)
    {
        boolean usesDb = dbRegistries.containsKey(regName);
        if(!usesDb)
        {
            super.refresh(regName);
        }
    }
    
    /** Late init method from Turbine Service model */
    public void init() throws InitializationException
    {
        if (hybrid)
        {
            super.init();
        }
    }

    /**
     * This is the shutdown method called by the
     * Turbine <code>Service</code> framework
     */
    public void shutdown()
    {
        if (hybrid)
        {
            super.shutdown();
        }
    }
    
    /**
     * @see javax.jms.MessageListener#onMessage(javax.jms.Message)
     */
    public void onMessage(Message message)
    {
        if(message instanceof ObjectMessage)
        {
            ObjectMessage objMsg = (ObjectMessage)message;
            Object obj = null;
            try
            {
                obj = objMsg.getObject();
            }
            catch (JMSException e)
            {
                logger.error("Failed to retrieve object from message.");
                return;
            }
            
            if((obj != null) && (obj instanceof RegistryMessage))
            {
	            RegistryMessage regMsg = (RegistryMessage)obj;
	            String entryName = regMsg.getEntryName();
	            String registry = regMsg.getRegistry();
	            if(RegistryMessage.ACTION_REFRESH.equals(regMsg.getAction()))
	            {
	                boolean usesDb = dbRegistries.containsKey(registry);
	                if(usesDb)
	                {
	                    LocalRegistry reg = (LocalRegistry) get(registry);
	                    reg.removeLocalEntry(entryName); //remove from cache
	                }
	                
                    refresh(registry);
	            }
            }
            else if(obj != null)
            {
                logger.error("Recieved unsupported object " + obj.getClass().getName());
            }
        }
        else
        {
            logger.error("Received unsupported message type ");
        }
    }
}
