/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.services.registry;

import java.util.Enumeration;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.jetspeed.services.registry.RegistryService;
import org.apache.jetspeed.om.registry.RegistryEntry;
import org.apache.jetspeed.om.registry.Registry;
import org.apache.jetspeed.util.ServiceUtil;
import org.apache.turbine.services.TurbineServices;
import org.apache.turbine.services.resources.ResourceService;
import org.apache.turbine.util.TurbineConfig;

/**
 * Reads all Registry files from the file system and imports them into Registry DB
 *
 * @author <a href="mailto:susinha@cisco.com">Suchisubhra Sinha</a>
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor</a>
 * @version $Id: RegistryImporter.java,v 1.6 2005/04/27 14:08:24 raphael Exp $
 */
public class RegistryImporter
{
    /**
     * Static initialization of the logger for this class
     */
    private static final JetspeedLogger logger =
        JetspeedLogFactoryService.getLogger("importer");
    protected boolean check = true;
    public RegistryImporter()
    {
    }
    public static void main(String args[])
    {
        logAndPrint("***** Registry Importer *****");
        boolean checkImport = true;
        //
        // initialize and bootstrap services
        //
        try
        {
            String root = "./webapp";
            String properties = "/WEB-INF/conf/RegistryImport.properties";
            TurbineConfig config = new TurbineConfig(root, properties);

            config.initialize();
            ResourceService serviceConf = ((TurbineServices)
                    TurbineServices.getInstance()).getResources(RegistryService.SERVICE_NAME);

        }
        catch (Throwable t)
        {
            String msg =
                "Registry Importer: error initializing Turbine configuration";
            logAndPrintError(msg, t);
            System.exit(1);
        }

        //
        // get a handle to the exporter service
        //
        RegistryService exporterService = getService("Registry", "Exporter");
        RegistryService importerService = getService("RegistryImporter", "Importer");

        if (exporterService.getClass().getName().equals(importerService.getClass().getName()))
        {
            String msg =
                "Registry Importer Error: Importer Class cannot equal Exporter Class.";
            logAndPrintError(msg, null);
            System.exit(1);
        }

        RegistryImporter importer = new RegistryImporter();
        boolean ran = importer.run(exporterService, importerService);
        logAndPrint("**** Registry Importer - completed");
        System.exit(0);
    }

    public boolean run(
        RegistryService exporterService,
        RegistryService importerService)
    {
        String msg;
        int registryCount = 0;
        int entryCount = 0;
        int skipCount = 0;
        try
        {
            msg = "Running with Importer Service: " + importerService.getClass();
            logAndPrint(msg);
            msg = "Running with Exporter Service: " + exporterService.getClass();
            logAndPrint(msg);
            Enumeration e = importerService.getNames();

            while (e.hasMoreElements())
            {
                String registryName = (String) e.nextElement();
                logAndPrint("Exporting Registry: " + registryName);
                Registry importRegistry = importerService.get(registryName);
                Registry exportRegistry = exporterService.get(registryName);

                Enumeration enum1 = exportRegistry.getEntries();
                while (enum1.hasMoreElements())
                {
                    RegistryEntry exportEntry = (RegistryEntry) enum1.nextElement();

                    if (registryName.equals(org.apache.jetspeed.services.Registry.SECURITY) ||
                        registryName.equals(org.apache.jetspeed.services.Registry.PORTLET))
                    {
                        if (null == importerService.getEntry(registryName, exportEntry.getName()))
                        {
                            importerService.addEntry(registryName, exportEntry);
                            logAndPrint("Imported into Registry: " + registryName + ", Entry = " + exportEntry.getName());
                            entryCount++;
                        }
                        else
                        {
                            if (importerService instanceof HybridRegistryService)
                            {
                                HybridRegistryService hybrid = (HybridRegistryService)importerService;
                                if (hybrid.getReplaceImportedEntries())
                                {
                                    importerService.addEntry(registryName, exportEntry);
                                    logAndPrint("Imported (update) into Registry: " + registryName + ", Entry = " + exportEntry.getName());
                                    entryCount++;
                                }
                                else
                                {
                                    logAndPrint("Skipping Entry, already exists: " + registryName + ", Entry = " + exportEntry.getName());
                                    skipCount++;
                                }
                            }
                        }
                    }
                }
                registryCount++;
            }
        }
        catch (Exception e)
        {
            logger.error("Error importing: ", e);
            e.printStackTrace();
            return false;
        }
        msg = "*** Registry Importer completed ***";
        logAndPrint(msg);
        msg = "Total Registries imported      : " + registryCount;
        logAndPrint(msg);
        msg = "Total Registry Entries imported: " + entryCount;
        logAndPrint(msg);
        msg = "Total Registry Entries skipped : " + skipCount;
        logAndPrint(msg);
        return true;
    }

    private static final void logAndPrint(String msg)
    {
        System.out.println(msg);
        logger.info(msg);
    }

    private static final void logAndPrintError(String msg, Throwable t)
    {
        System.err.println(msg);
        if (null == t)
        {
            logger.error(msg);
        }
        else
        {
            logger.error(msg, t);
        }
    }

    private static final RegistryService getService(String serviceName, String serviceMode)
    {
        RegistryService service = null;

        try
        {
            service = (RegistryService) ServiceUtil.getServiceByName(serviceName);
        }
        catch (org.apache.turbine.services.InstantiationException e)
        {
            String msg =
                "Registry Importer: error loading Registry " + serviceMode + " Service";
            logAndPrintError(msg, e);
            System.exit(1);
        }

        if (false == service.getInit())
        {
            String msg =
                "Registry Importer: error initializing Registry " + serviceMode + " Service";
            logAndPrintError(msg, null);
            System.exit(1);
        }
        return service;
    }
}
